$NOLIST			; switch off generation of the listing file
$NOMOD51		; switch off 8051 controller defaults and use...
$INCLUDE (..\asm\reg52.inc)	; ...another controller definition file (i.e. 8052).
$LIST			; switch on listing

;
;****************************************************************************
;   An Example Source Code Template for A51 Assembler Modules.
;   Modify/adapt the names (in lower case letters) to your own needs.
;****************************************************************************
;
; This code template gives ASSEMBLER novices an idea of how to use the A51 
; Assembler.  It's not necessary to build each module in this way.  All 
; entries except the END statement at the "end of the file", are optional.  
; Remove any unused segment declarations that do not contain not variable 
; space or valid assembler instructions.  Also keep in mind that this one 
; file cannot demonstrate all possible variations of A51 operations.  The 
; user is admonished to read all appropriate manual sections for more details.
;
;----------------------------------------------------------------------------
;   Give the module a name (optional)
NAME		module_name
;----------------------------------------------------------------------------
;
; Here we import the symbols from other modules.  You can use up to 256 
; external symbols in one module.
;
; Examples:
EXTRN	CODE   (code_symbol)    ; can be a sub-routine entry declaration in a
				; CODE segment or used with a CODE directive.

EXTRN	DATA   (data_symbol)	; can be any symbol declared in DATA segments
				; or used with the DATA directive.

EXTRN	BIT    (bit_symbol)	; can be any symbol declared in BIT segments
				; or used with the BIT directive.

EXTRN	XDATA  (xdata_symbol)	; can be any symbol declared in XDATA segments
				; or used with the XDATA directive.

EXTRN   NUMBER (typeless_symbol); can be any symbol declared with an EQU or 
				; SET directive
;
; You can also state more than one symbol in one EXTRN statement.  For example:
;
EXTRN  	CODE (sub_routine1, sub_routine2), DATA (variable_1)
;----------------------------------------------------------------------------
$EJECT
;----------------------------------------------------------------------------
; Here you can export symbols to other modules.  You may have up to 256 PUBLIC 
; symbols IN ONE MODULE.
;
; For example:
PUBLIC	data_variable
PUBLIC  code_entry		
PUBLIC  typeless_number
PUBLIC  xdata_variable
PUBLIC  bit_variable
;
; You can also declare more than one symbol in an EXTRN statement.  
; For example:
PUBLIC  data_variable1, code_table, typeless_num1, xdata_variable1
;
;----------------------------------------------------------------------------
;
; Insert the STACK segment in the main module here
STACK		SEGMENT	IDATA		; STACK Always lies in IDATA RAM.
		RSEG	STACK		; switch to STACK segment.
		DS	5		; reserve your stack space
					; 5 bytes in this example.
;
;----------------------------------------------------------------------------
$EJECT
;----------------------------------------------------------------------------
; Insert segment and variable declarations here.  You may have up to 255
; segments in each module
;
; Examples:
; DATA SEGMENT:  reserves space in DATA RAM (delete segment if not used!)
data_seg_name	SEGMENT	DATA		; segment for DATA RAM.
		RSEG	data_seg_name	; switch to this data segment
data_variable:	DS	1		; reserve 1 Bytes for data_variable
data_variable1:	DS	2		; reserve 2 Bytes for data_variable1
;
; XDATA SEGMENT:  reserves space in XDATA RAM (delete segment if not used!)
xdata_seg_name	SEGMENT XDATA		; segment for XDATA RAM
		RSEG	xdata_seg_name	; switch to this xdata segment
xdata_variable:	DS	1		; reserve 1 Byte for an xdata_variable
xdata_array:	DS	500		; reserve 500 Bytes for an xdata_array
;
; INPAGE XDATA SEGMENT:  reserves space in XDATA RAM page (page size: 256 Bytes)
; INPAGE segments are useful for @R0 addressing methodes. 
; (delete segment if not used!)
page_xdata_seg	SEGMENT XDATA INPAGE	; INPAGE segment for XDATA RAM
		RSEG	xdata_seg_name	; switch to this xdata segment
xdata_variable1:DS	1		; reserve 1 Byte for xdata_variable1
;
; ABSOLUTE XDATA SEGMENT:  reserves space in XDATA RAM at absolute addresses.
; ABSOLUTE segments are useful for memory mapped I/O.
; (delete segment if not used!)
		XSEG	AT 8000H        ; switch absolute XDATA segment @8000H
XIO:           	DS	1		; reserve 1 Byte for XIO port
XCONFIG:	DS	1		; reserve 1 Byte for XCONFIG port
;
; BIT SEGMENT:  reserves space in BIT RAM (delete segment if not used!)
bit_seg_name	SEGMENT	BIT		; segment for BIT RAM.
		RSEG	bit_seg_name	; switch to this bit segment
bit_variable:	DBIT	1		; reserve 1 Bit for bit_variable
bit_variable1:	DBIT	4		; reserve 4 Bits for bit_variable1
;
;----------------------------------------------------------------------------
;
; Add any constant (typeless) numbers here.
;
; Examples:
typeless_number	EQU	0DH			; assign 0D hex
typeless_num1	EQU	typeless_number-8	; evaluate typeless_num1
;
;----------------------------------------------------------------------------
$EJECT
;****************************************************************************
;
; Provide an LJMP to start at reset address (address 0) in the main module.
; In this same style you can declare any interrupt functions (see later in
; the template file).
;
		CSEG	AT	0	; absolute Code SEGment AT address 0
		LJMP	start		; install RESET location (jump to start)
;
;----------------------------------------------------------------------------
;
; CODE SEGMENT: reserves space in CODE ROM for assembler instructions
code_seg_name	SEGMENT	CODE
	
		RSEG	code_seg_name	; switch to this code segment

		USING	0		; state register_bank used
					; for the following program code.  

start:		MOV	SP,#STACK-1	; assign stack at beginning
;
;	insert your own assembly here code
;---------------------------------------------------------------------------
;       examples:	(Note:  this code itself is just nonsense!)
		ORL	IE,#82H 	; enable interrupt system (timer 0)
		SETB	TR0		; enable timer 0

repeat_label:	MOV	A,data_symbol
		ADD	A,#typeless_symbol
		CALL	code_symbol
		MOV	DPTR,#xdata_symbol
		MOVX	A,@DPTR
		MOV	R1,A
		PUSH	AR1
		CALL	sub_routine1
		POP	AR1
		ADD	A,R1
		JMP	repeat_label

code_entry:     CALL	code_symbol
		RET

code_table:	DW	repeat_label
		DW	code_entry
		DB	typeless_number
		DB	0
;		
;----------------------------------------------------------------------------
$EJECT
;****************************************************************************
;
; To place interrupt routines, put an LJMP at the interrupt vector address.
;
; Example:
		CSEG	AT  0BH		; 0BH is address for Timer 0 interrupt
		LJMP	timer0int
;
;---------------------------------------------------------------------------
;
; Give each interrupt function its own code segment
;
; Example:
int0_code_seg	SEGMENT	CODE		; segment for interrupt function
		RSEG	int0_code_seg	; switch to this code segment
		USING	1		; register bank for interrupt routine
;
timer0int:	PUSH	PSW
		MOV	PSW,#08H	; register bank 1
		PUSH	ACC
		MOV	R1,data_variable
		MOV	DPTR,#xdata_variable
		MOVX	A,@DPTR
		ADD	A,R1
		MOV	data_variable1,A
		CLR	A
		ADD	A,#0
		MOV	data_variable1+1,A
		POP	ACC
		POP	PSW
		RETI
;		
;---------------------------------------------------------------------------
;
		END		; End Of File
;
; Use this little example in good health!.......
;
